@@ -314,7 +314,7 @@ goc_group_class_init (GocItemClass *item_klass)
 static void
 goc_group_init (GocGroup *group)
 {
-	group->priv = g_malloc0 (sizeof (struct GocGroupPriv));
+	group->priv = g_new0 (struct GocGroupPriv, 1);
 	group->priv->children = g_ptr_array_new ();
 }
 
@@ -387,7 +387,7 @@ static void
 goc_group_fake_xchildren (GocGroup *group)
 {
 	g_list_free (group->Xchildren);
-	group->Xchildren = group->priv->children
+	group->Xchildren = group->priv->children->len
 		? g_list_prepend (NULL, goc_group_get_child (group, 0))
 		: NULL;
 }
