@@ -108,18 +108,21 @@ go_option_menu_detacher (GtkWidget *widget, GtkMenu *menu)
 static void
 go_option_menu_update_contents (GOOptionMenu *option_menu)
 {
-	const char *text;
 	GtkWidget *w;
 
 	g_return_if_fail (GO_IS_OPTION_MENU (option_menu));
 
 	w = gtk_bin_get_child (GTK_BIN (option_menu->selected));
-	if (GTK_IS_LABEL (w))
-		text = gtk_label_get_text (GTK_LABEL (w));
-	else
-		text = "";
+	if (GTK_IS_LABEL (w)) {
+		GtkLabel *src = GTK_LABEL (w);
+		GtkLabel *dst = option_menu->button_label;
 
-	gtk_label_set_text (option_menu->button_label, text);
+		gtk_label_set_text (dst, gtk_label_get_label (src));
+		gtk_label_set_use_markup (dst, gtk_label_get_use_markup (src));
+		gtk_label_set_ellipsize (dst, gtk_label_get_ellipsize (src));
+	} else {
+		gtk_label_set_text (option_menu->button_label, NULL);
+	}
 }
 
 void
@@ -392,6 +395,17 @@ go_option_menu_get_history (GOOptionMenu *option_menu)
 	return GTK_WIDGET (option_menu->selected);
 }
 
+/**
+ * go_option_menu_get_label:
+ * @option_menu: #GOOptionMenu
+ *
+ * Returns: (transfer none): The #GtkLabel used for the button text.
+ */
+GtkWidget *
+go_option_menu_get_label (GOOptionMenu *option_menu)
+{
+	return GTK_WIDGET (option_menu->button_label);
+}
 
 static void
 go_option_menu_set_property (GObject            *object,
@@ -501,13 +515,14 @@ go_option_menu_init (GOOptionMenu *option_menu)
 	option_menu->button_label = GTK_LABEL (gtk_label_new (""));
 	gtk_box_pack_start (box, GTK_WIDGET (option_menu->button_label),
 			    FALSE, TRUE, 0);
+	g_object_set (option_menu->button_label, "xalign", 0.0, NULL);
 
 	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE);
 	g_object_set (arrow, "xalign", 0.75, NULL);
 	gtk_box_pack_end (box, arrow, FALSE, FALSE, 0);
 
 	sep = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
-	gtk_box_pack_end (box, sep, FALSE, FALSE, 0);
+	gtk_box_pack_end (box, sep, FALSE, FALSE, 6);
 
 	gtk_container_add (GTK_CONTAINER (option_menu), GTK_WIDGET (box));
 }
